from __future__ import annotations

import datetime
import os
import uuid
from typing import Protocol

import httpx
import pydantic
import state
from PyQt6.QtCore import QPoint, Qt
from PyQt6.QtGui import QAction, QDragEnterEvent, QDragMoveEvent, QDropEvent, QIcon
from PyQt6.QtWidgets import (
    QHBoxLayout,
    QListWidget,
    QListWidgetItem,
    QMenu,
    QMessageBox,
    QPushButton,
    QVBoxLayout,
    QWidget,
)
from request_client import RequestClient


class DisplayProtocol(Protocol):
    def name(self) -> str:
        raise NotImplementedError

    def delete(self) -> None:
        raise NotImplementedError

    def details(self) -> QWidget:
        raise NotImplementedError


class File(pydantic.BaseModel):
    file_id: uuid.UUID
    file_name: str
    file_size: int
    sha512: str
    created_at: datetime.datetime
    updated_at: datetime.datetime

    def name(self) -> str:
        return self.file_name

    def delete(self) -> None:
        RequestClient().client.delete("/files", params={"file_id": self.file_id})

    def details_button(self) -> QPushButton:
        # TODO
        raise NotImplementedError


class Folder(pydantic.BaseModel):
    folder_id: uuid.UUID
    owner_id: int
    folder_name: str
    created_at: datetime.datetime

    def name(self) -> str:
        return self.folder_name

    def delete(self) -> None:
        RequestClient().client.delete("/folders", params={"folder_id": self.folder_id})

    def details_button(self) -> QPushButton:
        # TODO
        raise NotImplementedError


class ListResponse(pydantic.BaseModel):
    folder_id: uuid.UUID
    files: list[File]
    folders: list[Folder]

    def items(self) -> list[DisplayProtocol]:
        return self.files + self.folders


class FileListWidget(QListWidget):
    def __init__(self, state: state.State):
        super().__init__()
        self.state = state
        self.setAcceptDrops(True)
        self.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
        self.customContextMenuRequested.connect(self.show_context_menu)

        self.responses: list[ListResponse] = []
        self.update_response()

    def current_response(self) -> ListResponse:
        if not self.responses:
            self.update_response()
        return self.responses[-1]

    def update_response(self):
        if not self.responses:
            response = ListResponse.model_validate_json(
                RequestClient().client.get("/folders").text
            )
            self.responses.append(response)
            print(response.files)
        self.responses[-1] = ListResponse.model_validate_json(
            RequestClient()
            .client.get("/folders", params={"folder_id": self.responses[-1].folder_id})
            .text
        )
        self.update()

    def dragEnterEvent(self, event: QDragEnterEvent):
        if event.mimeData().hasUrls():
            event.accept()
        else:
            event.ignore()

    def dragMoveEvent(self, e: QDragMoveEvent | None) -> None:
        return self.dragEnterEvent(e)

    def dropEvent(self, event: QDropEvent):
        event.accept()
        print("hi")
        for url in event.mimeData().urls():
            file_path = url.toLocalFile()
            self.upload_file(file_path)

    def upload_file(self, file_path):
        file_name = os.path.basename(file_path)
        try:
            with open(file_path, "rb") as f:
                files = {"file": (file_name, f)}
                response = RequestClient().client.post(
                    "http://localhost:3000/files",
                    files=files,
                    params={"parent_folder": self.current_response().folder_id},
                )
                if response.is_success:
                    QMessageBox.information(
                        self, "Success", "File uploaded successfully"
                    )
                    self.update_response()
                else:
                    QMessageBox.warning(
                        self, "Error", f"Upload failed: {response.text}"
                    )
        except httpx.HTTPError as e:
            QMessageBox.critical(self, "HTTP Error", str(e))

    def add_file_item(self, file_name):
        item = QListWidgetItem(file_name)
        item.setIcon(QIcon.fromTheme("text-x-generic"))  # File icon
        self.addItem(item)

    def show_context_menu(self, pos: QPoint):
        item = self.itemAt(pos)
        if item:
            menu = QMenu(self)
            details_action = QAction("Details", self)
            details_action.triggered.connect(lambda: self.show_details(item))
            delete_action = QAction("Delete", self)
            delete_action.triggered.connect(lambda: self.delete_item(item))
            menu.addAction(details_action)
            menu.addAction(delete_action)
            menu.exec(self.mapToGlobal(pos))

    def show_details(self, item):
        file_name = item.text()
        QMessageBox.information(self, "Details", f"Details for {file_name}")

    def delete_item(self, item):
        row = self.row(item)
        item = self.current_response().items()[row]
        item.delete()
        self.update_response()
        QMessageBox.information(self, "Delete", f"{item.name()} deleted")

    def update(self) -> None:
        self.clear()
        last = self.responses[-1]
        for item in last.items():
            self.add_file_item(item.name())


class Sidebar(QWidget):
    def __init__(self, state: state.State):
        super().__init__()
        self.state = state
        layout = QVBoxLayout()
        self.setLayout(layout)
        # Add your sidebar buttons here
        for i in range(5):  # Example buttons
            btn = QPushButton(f"Button {i+1}")
            layout.addWidget(btn)
        layout.addStretch()


class MainFileWidget(QWidget):
    def __init__(self, state: state.State):
        super().__init__()
        self.state = state
        layout = QHBoxLayout()
        self.sidebar = Sidebar(state)
        self.file_list = FileListWidget(state)
        layout.addWidget(self.sidebar)
        layout.addWidget(self.file_list)
        self.setLayout(layout)
